#include <stdlib.h>
#include <string.h>

#include "./game_highscores.h"
#include "./vec.h"
#include "./json_reader.h"
#include "./json_printer.h"
#include "./memtools.h"
#include "./reader.h"

#define HIGHSCORE_PATH "highscores.json"

void highscores_free(Vec* highscores) {
  for (size_t i = 0; i < vec_length(highscores); ++i) {
    Highscore* hs = *vec_at(highscores, i);
    free(hs->name);
    free(hs);
  }
  vec_free(highscores);
}

JsonValue* uri_to_json(const char* uri) {
  char command[255];
  sprintf(command, "curl -X GET %s 2>/dev/null", uri);

  FILE* curl_output_file = popen(command, "r");
  char* curl_output = readfile(curl_output_file);
  pclose(curl_output_file);

  reader* r = make_reader(curl_output);
  JsonValue* json = parse_json(r);
  free(r);
  free(curl_output);

  return json;
}
/* `popen` runs a command in the shell and returns the commands stdout file. */

JsonValue* download_highscore_json(void) {
  return uri_to_json("https://stream.inpro.informatik.uni-freiburg.de/highscores");
}

JsonValue* download_highscore_json_for(char* name) {
  char uri[255];
  sprintf(uri, "https://stream.inpro.informatik.uni-freiburg.de/highscores/%s", name);
  return uri_to_json(uri);
}

void upload_highscore_json(JsonValue* highscore) {
  /* Write highscore to a temporary json file. */
  FILE* tmp_file = fopen("tmp_highscore.json", "w");
  json_fprint_value(tmp_file, 0, highscore);
  fclose(tmp_file);

  /* Send the json file to the highscore server via a HTTP POST request. */
  const char* command =
    "curl -X POST -H 'Content-Type: application/json' "
    "-d @tmp_highscore.json "
    "https://stream.inpro.informatik.uni-freiburg.de/highscores "
    "2>&1";
  FILE* curl_output = popen(command, "r");
  pclose(curl_output);

  /* Delete the temporary file. */
  remove("tmp_highscore.json");
}

Vec* json_to_highscores(JsonValue* highscores_json) {
  if (json_value_type(highscores_json) != JSON_OBJECT) {
    return NULL;
  }

  Vec* highscores = vec_new();
  Vec* members = json_object_members(json_value_as_object(highscores_json));
  for (size_t i = 0; i < vec_length(members); ++i) {
    JsonMember* m = *vec_at(members, i);
    JsonValue* v = json_member_value(m);
    if (json_value_type(v) != JSON_OBJECT) {
      highscores_free(highscores);
      return NULL;
    }

    JsonObject* o = json_value_as_object(v);
    Highscore* highscore = malloc(sizeof(Highscore));

    // Read Name Member
    {
      JsonValue* v = json_object_find(o, "name");
      if (v == NULL || json_value_type(v) != JSON_STRING) {
        highscores_free(highscores);
        free(highscore);
        return NULL;
      }
      highscore->name = strcpy_malloc(json_value_as_string(v));
      if (highscore->name == NULL) {
        highscores_free(highscores);
        free(highscore);
        return NULL;
      }
    }

    // Read Points Member
    {
      JsonValue* v = json_object_find(o, "points");
      if (v == NULL || json_value_type(v) != JSON_NUMBER) {
        highscores_free(highscores);
        free(highscore);
        return NULL;
      }
      highscore->points = json_value_as_number(v);
    }

    // Read Distance Member
    {
      JsonValue* v = json_object_find(o, "distance");
      if (v == NULL || json_value_type(v) != JSON_NUMBER) {
        highscores_free(highscores);
        free(highscore);
        return NULL;
      }
      highscore->distance = json_value_as_number(v);
    }

    vec_push(highscores, highscore);
  }
  return highscores;
}

JsonValue* highscore_to_json(Highscore* highscore) {
  Vec* members = vec_new();

  JsonValue* name_val = json_value_new_string(strcpy_malloc(highscore->name));
  JsonMember* name_member = json_member_new(strcpy_malloc("name"), name_val);
  vec_push(members, name_member);

  JsonValue* points_val = json_value_new_number(highscore->points);
  JsonMember* points_member = json_member_new(strcpy_malloc("points"), points_val);
  vec_push(members, points_member);

  JsonValue* distance_val = json_value_new_number(highscore->distance);
  JsonMember* distance_member = json_member_new(strcpy_malloc("distance"), distance_val);
  vec_push(members, distance_member);

  return json_value_new_object(json_object_new(members));
}
